@@ -1,8 +1,8 @@
 package HTML::Template::Compiled;
-# $Id: Compiled.pm 1056 2008-06-16 21:10:07Z tinita $
+# $Id: Compiled.pm 1075 2008-08-10 16:55:31Z tinita $
 # doesn't work with make tardist
 #our $VERSION = ($version_pod =~ m/^\$VERSION = "(\d+(?:\.\d+)+)"/m) ? $1 : "0.01";
-our $VERSION = "0.91_003";
+our $VERSION = "0.92_001";
 use Data::Dumper;
 BEGIN {
 use constant D => $ENV{HTC_DEBUG} || 0;
@@ -850,6 +850,7 @@ sub init {
         #use_expressions        => 0,
         use_perl               => 0,
         open_mode              => '',
+        no_includes            => 0,
         %args,
     );
     $self->set_loop_context(1) if $args{loop_context_vars};
@@ -906,6 +907,9 @@ sub init {
             }
         });
     }
+    if ($defaults{no_includes}) {
+        $parser->remove_tags(qw/ INCLUDE INCLUDE_VAR INCLUDE_STRING /);
+    }
     $self->set_parser($parser);
     my $compiler = $self->compiler_class->new;
     $self->set_compiler($compiler);
@@ -1141,6 +1145,7 @@ sub new_from_object {
     $new->set_path($path);
     $new->set_perl(undef);
     if (my $cached = $new->from_cache) {
+        $cached->set_plugins($self->get_plugins);
         $cached->init_includes;
         return $cached
     }
@@ -1531,7 +1536,7 @@ HTML::Template::Compiled - Template System Compiles HTML::Template files to Perl
 
 =head1 VERSION
 
-$VERSION = "0.91_003"
+$VERSION = "0.92_001"
 
 =cut
 
@@ -2390,6 +2395,12 @@ browse through the stack.
 Now everything will be escaped for HTML unless you explicitly specify C<ESCAPE=0> (no escaping)
 or C<ESCAPE=URL>.
 
+=item no_includes (since 0.92)
+
+Default is 0. If set to 1, the tags INCLUDE, INCLUDE_VAR and INCLUDE_STRING
+will cause a template syntax error when creating. This can be useful when opening
+untrusted templates, otherwise any file in the filesystem could be opened.
+
 =item debug_file (fixed) (since 0.91_001)
 
 Additionally to the context_vars __filename__ and __filenameshort__ you
@@ -2580,7 +2591,7 @@ tt-style and your own C<{{}} > style, then say:
         ],
     );
 
-=item use_expressions
+=item use_expressions (since 0.91_003)
 
 Set to 1 if you want to use expressions. They work more or less like
 in L<HTML::Template::Expr> - I took the parsing code from it and
@@ -2857,9 +2868,9 @@ on speed.
 
 Setting case_sensitive to 1, loop_context_vars to 0 and global_vars to 0 saves time.
 
-On the other hand, compared to HTML::Template, the speed gain is biggest (under mod_perl
-you save ca. 86%, under CGI about 10%), if you use case_sensitive = 1, loop_context_vars = 0,
-global_vars = 1.
+On the other hand, compared to HTML::Template, you have a large speed gain under mod_perl
+(you save ca. 87%), if you use case_sensitive = 1, loop_context_vars = 0,
+global_vars = 1. With CGI HTC is slower (about 1 1/2 times).
 
 See the C<examples/bench.pl> contained in this distribution.
 
@@ -2892,7 +2903,7 @@ If I can efficiently use it, it was worth it.
 
 =head1 RESOURCES
 
-See http://htcompiled.sf.net/ for current releases not yet on CPAN and for cvs access.
+See http://htcompiled.sf.net/ for svn access.
 
 =head1 SEE ALSO
 
